︠4927baf1-d96b-481e-854a-436ed2d38c3es︠
from sage.plot.arc import Arc

sage: P = UnsignedInfinityRing
unsigned_oo=P.0


mu = var('mu')

torus = [0, 1, 1, mu, mu, unsigned_oo, unsigned_oo, 0]#each pair of numbers is a side of the fundamental domain: hyperbolic line from 0 to 1, 1 to mu, mu to infinity, and infinity to 0.

#load text files
with open('closed_geodesics_5.txt', 'r') as g:
    closed = [item2.strip() for item2 in g]
g.close()
with open('simple_geodesics_4.txt', 'r') as f:
    simple_closed = [item1.strip() for item1 in f]
f.close()

#returns graphics object for hyperbolic line with endpoints endpt1 and endpt2 (in the boundary of H), with given color, and zorder.
def graphic_in_H(endpt1, endpt2, color, some_zorder):
    if endpt1 == unsigned_oo or endpt2 == unsigned_oo:#vertical line case
        if endpt1 == unsigned_oo:
            finite_one = endpt2
        else:
            finite_one = endpt1
        a_graphic = line([(finite_one,0),(finite_one, 100)], rgbcolor = color, zorder = some_zorder)
    else:
        a_graphic = arc(((endpt1+endpt2)/2,0), abs(endpt1-endpt2)/2, sector = (0,pi), rgbcolor = color, zorder= some_zorder)
    return a_graphic

def get_arc_data(z1, z2):
    t = abs(z1-z2)*(1+norm(z1-z2)/4-norm(z1+z2)/4)/(I*(z1*conjugate(z2)-z2*conjugate(z1)))#this is the result of a pencil and paper computation. For the circle thru z1 and z2 to intersect the unit circle at right angle, you need r^2+1 = |c|^2 by pythagorean theorem. Combine this with the facts that |z1-r|=c and c is on perp bisector of z1 and z2 [parametrization: (z1+z2)/2+(I*(z1-z2)/abs(z1-z2))*t], solve for t
    c = (z1+z2)/2+(I*(z1-z2)/abs(z1-z2))*t#c is center
    r = abs(z1-c)#r is the radius
    return [c, r]


def graphic_in_D(endpt1, endpt2, some_color, some_zorder): #same as graphics_in_H, but in disk model: takes two endpoints on unit circle and returns the graphics object for the hyperbolic line with those endpoints, given color and zorder
    z1 = endpt1
    z2 = endpt2
    if abs(imag_part(z1/z2)) <= 10^-15:#straight line case
        next_graphic = line([(real_part(z1), imag_part(z1)), (real_part(z2), imag_part(z2))], rgbcolor = some_color)
        test_next_graphic = next_graphic[0]
        test_next_graphic.set_zorder(some_zorder)
    else:
        data = get_arc_data(z1, z2)
        c = data[0]
        r = data[1]
        gamma1 = min(arg(z1-c), arg(z2-c))#ensures that minor arc (one that is inside the disc) is drawn, not the arc that's outside disc
        gamma2 = max(arg(z1-c), arg(z2-c))
        if(gamma2-gamma1 > pi):
            next_graphic = arc((real_part(c),imag_part(c)), r, sector = (gamma2, gamma1+2*pi), rgbcolor = some_color, zorder = some_zorder)
        else:
            next_graphic = arc((real_part(c),imag_part(c)), r, sector = (gamma1, gamma2), rgbcolor = some_color, zorder = some_zorder)
    return next_graphic


def h_to_d(z):#maps H to D in such a way that infinity is mapped to -I, 0 is mapped to -1, and 1 is mapped to I. That way, if you let the 4th vertex be 1, you get nice symmetry.
    if z == unsigned_oo:
        return -I
    elif abs(z+1+I) <= 10^-15:
        return unsigned_oo
    else:
        return (-I*conjugate(z)+1+I)/(conjugate(z)-1-I)

def d_to_h(z):#the inverse of h_to_d
    if abs(z + I) == 10^-15:
        return unsigned_oo
    else:
        return (1-I)*(conjugate(z)+1)/(conjugate(z)-I)

@interact
def draw_geodesics_interactive(model = selector( ['disk', 'half-plane'], label = "Model:"), num_geodesics = input_grid(1, 2, default=[[int(len(closed)/2),int(len(simple_closed)/2)]], label = "number of closed, simple closed geodesics to draw, respectively")):
    if model == 'disk':
        @interact
        def draw_geodesics_in_D(mu_angle = slider(-85, 85, 5, 0, label = 'position of 4th vertex (on unit circle, in degrees)')):
            mu_comp = float(cos(mu_angle*pi/180))+I*float(sin(mu_angle*pi/180))
            mu_value = d_to_h(mu_comp)#since the elements of the text files are expressions for the endpoints in H, convert back to find 4th vertex in H
            for w in range(0,8,2):
                next_graphic = graphic_in_D(h_to_d(torus[w].subs(mu == mu_value)), h_to_d(torus[w+1].subs(mu == mu_value)), (1,0,0), 3)#get graphics object for each side of fundamental domain
                center_radius = get_arc_data(h_to_d(torus[w].subs(mu == mu_value)), h_to_d(torus[w+1].subs(mu == mu_value)))
                if w == 0:
                    torus_graphic_sum = next_graphic
                    obscure_graphic_sum = circle((real_part(center_radius[0]), imag_part(center_radius[0])), center_radius[1], thickness = 0, fill = True, facecolor = 'white', zorder = 2)#to only show the stuff inside the fundamental domain, simply cover up everything else with white sections of circles
                else:
                    torus_graphic_sum += next_graphic
                    obscure_graphic_sum += circle((real_part(center_radius[0]), imag_part(center_radius[0])), center_radius[1], thickness = 0, fill = True, facecolor = 'white', zorder = 2)

            unit_circle = circle((0,0), 1, edgecolor = 'black', thickness = 1, zorder = 4)

            for q in range(0, 2*num_geodesics[0][0], 2):

                #the 2nth and (2n+1)th expressions in the text file is a pair of endpoints of the (n+1)th geodesic in H, in terms of mu. evaluate at mu = mu_value, convert to D, then add hyperbolic line with those points as its endpoints to closed_graphic_sum
                if q == 0:
                    closed_graphic_sum = graphic_in_D(h_to_d(sage_eval(closed[0], locals={'mu':mu_value})), h_to_d(sage_eval(closed[1], locals={'mu':mu_value})), (0,0,1), 1)
                else:
                    closed_graphic_sum += graphic_in_D(h_to_d(sage_eval(closed[q], locals={'mu':mu_value})), h_to_d(sage_eval(closed[q+1], locals={'mu':mu_value})), (0,0,1), 1)

            print "Here is an image of the first",
            print num_geodesics[0][0],
            print "closed geodesics:"
            show(closed_graphic_sum+unit_circle+torus_graphic_sum+obscure_graphic_sum, axes = false, xmin = -1, xmax = 1, ymin = -1, ymax = 1)

            for p in range(0, 2*num_geodesics[0][1], 2):

                if p == 0:
                    simple_graphic_sum = graphic_in_D(h_to_d(sage_eval(simple_closed[0], locals={'mu':mu_value})), h_to_d(sage_eval(simple_closed[1], locals={'mu':mu_value})), (0,0,1), 1)
                else:
                    simple_graphic_sum += graphic_in_D(h_to_d(sage_eval(simple_closed[p], locals={'mu':mu_value})), h_to_d(sage_eval(simple_closed[p+1], locals={'mu':mu_value})), (0,0,1), 1)

            print "Here is an image of the first",
            print num_geodesics[0][1],
            print "simple closed geodesics:"
            show(simple_graphic_sum+unit_circle+torus_graphic_sum+obscure_graphic_sum, axes = false, xmin = -1, xmax = 1, ymin = -1, ymax = 1)

            return
        #end draw_geodesics_in_D

    elif model == 'half-plane':
        @interact
        def draw_geodesics_in_H(mu_value = slider(1.2, 10.0, .2, float(2), label = 'x-coordinate of 4th vertex')):
            @interact
            def with_zoom(zoom=range_slider(-1*mu_value, 2*mu_value, default=(0,mu_value))):

                for i in range(0,8,2):
                    if i == 0:
                        torus_graphic_sum = graphic_in_H(torus[i].subs(mu == mu_value), torus[i+1].subs(mu == mu_value), (1,0,0), 3)
                    else:
                        torus_graphic_sum += graphic_in_H(torus[i].subs(mu == mu_value), torus[i+1].subs(mu == mu_value), (1,0,0), 3)

                #create white rectangles and semi-circles that cover up the areas outside of the fundamental domain--the largest x or y coordinate possible is 30, so 40 ensures it extends to the edges of the screen, even when zoomed all the way out
                left_rect = polygon([[-40, -40],[0, -40], [0, 40], [-40, 40]], rgbcolor = (1,1,1))
                left_rect_test = left_rect[0]
                left_rect_test.set_zorder(2)
                right_rect = polygon([[mu_value, -40],[40, -40], [40, 40], [mu_value, 40]], rgbcolor = (1,1,1))
                right_rect_test = right_rect[0]
                right_rect_test.set_zorder(2)
                left_circle = circle((1/2, 0), 1/2, rgbcolor = (1,1,1), facecolor = (1,1,1), fill = True, zorder = 2)
                right_circle = circle(((mu_value+1)/2,0), (mu_value-1)/2, rgbcolor = (1,1,1), facecolor = (1,1,1), fill = True, zorder = 2)
                obscure_graphics = left_rect+right_rect+left_circle+right_circle

                axis = line([(-40,0),(40,0)], rgbcolor = (0,0,0), thickness = 1.5, zorder = 4)


                print "Below is an image of the first",
                print num_geodesics[0][0],
                print "closed geodesics:"
                for j in range(0,2*num_geodesics[0][0],2):
                    if j == 0:
                        closed_graphic_sum = graphic_in_H(sage_eval(closed[j], locals = {'mu':mu_value}), sage_eval(closed[j+1], locals = {'mu':mu_value}), (0,0,1), 1)
                    else:
                        closed_graphic_sum += graphic_in_H(sage_eval(closed[j], locals = {'mu':mu_value}), sage_eval(closed[j+1], locals = {'mu':mu_value}), (0,0,1), 1)

                show(closed_graphic_sum+torus_graphic_sum+obscure_graphics+axis, xmin = zoom[0], xmax = zoom[1], ymin = 0, ymax = 3/2*zoom[1], axes = True)


                print "Below is an image of the first",
                print num_geodesics[0][1],
                print "simple closed geodesics:"
                for k in range(0,2*num_geodesics[0][1],2):
                    if k == 0:
                        simple_closed_graphic_sum = graphic_in_H(sage_eval(simple_closed[k], locals = {'mu':mu_value}), sage_eval(simple_closed[k+1], locals = {'mu':mu_value}), (0,0,1), 1)
                    else:
                        simple_closed_graphic_sum += graphic_in_H(sage_eval(simple_closed[k], locals = {'mu':mu_value}), sage_eval(simple_closed[k+1], locals = {'mu':mu_value}), (0,0,1), 1)

                show(simple_closed_graphic_sum+torus_graphic_sum+obscure_graphics+axis, xmin = zoom[0], xmax = zoom[1], ymin = 0, ymax = 3/2*zoom[1], axes = True)


                return
            return
    return
︡56ff05c1-86c3-448f-b07a-865d1a9315a2︡{"interact":{"controls":[{"button_classes":null,"buttons":false,"control_type":"selector","default":0,"label":"Model:","lbls":["disk","half-plane"],"ncols":null,"nrows":null,"var":"model","width":null},{"control_type":"input-grid","default":[["518","98"]],"label":"number of closed, simple closed geodesics to draw, respectively","ncols":2,"nrows":1,"var":"num_geodesics","width":5}],"flicker":false,"id":"9a93868f-dc50-47df-9cd8-494f5b7b0b48","layout":[[["model",12,null]],[["num_geodesics",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡










